from .list import ListView
from .create import CreateView
from .update import UpdateView, MassUpdateView
from .detail import DetailView
from .delete import DeleteView, MassDeleteView
from .duplicate import DuplicateView
from .filter import FilterView
from .base import ModuleView
